#include <u.h>
#include <libc.h>
#include <ctype.h>

static char* 	plus2space(char *s);
static int 		qsdec(char *qs);
static char *	urlunesc(char *s);

static void
usage(void)
{	print("usage: qsparse query\n");
	exits("usage");
}


void main(int argc, char *argv[])
{
	ARGBEGIN{
	default: usage();
	}ARGEND

	if(argc != 1)
		usage();
	qsdec(argv[0]);
}

/*
*	the step is:
*	1.	&
*	2.	=
*	3.	+
*	4.	%XX
*	-Kenar-
*/
static int
qsdec(char *qs)
{
	char **q;
	char *p,*s, *t;
	int n, i;
	/*	count '&' in the string	*/

	n = 0;
	for(s = qs; *s; s++)
		if(*s == '&')
			n++;
	q = calloc(n + 1,sizeof(char*));
	/*	getfields overwrite the string, so I must ...	*/
	s = strdup(qs);
	n = getfields(s, q, n+1, 0, "&");
	for(i = 0; i < n; i++){
		p = strchr(q[i], '=');
		if(p){
			*p++ = 0;
			s = malloc(p - q[i] + 5);
			strcpy(s,"QS_");
			strcat(s,q[i]);
			/* translate and check the the value before put env */
			s = plus2space(s);
			p = plus2space(p);
			s = urlunesc(s);
			p = urlunesc(p);
			for(t= s; *t; t++)
				if(!isalnum(*t) && !strchr("_", *t))
					return -1;
			putenv(s, p);
		}
		else
			return -1;
	}
	return 0;
}

char *
plus2space(char *s)
{
	char *p;
	for(p = s; *p; p++)
		if(*p == '+')
			*p = ' ';
	return s;
}

/* go from possibly-latin1 url with escapes to utf
*	stolen from wiki
*	-Kenar-	*/
static char *
urlunesc(char *s)
{
	char *t, *v, *u;
	Rune r;
	int c, n;

	/* unescape */
	u = malloc(strlen(s)+1);
	for(t = u; c = *s; s++){
		if(c == '%'){
			n = s[1];
			if(n >= '0' && n <= '9')
				n = n - '0';
			else if(n >= 'A' && n <= 'F')
				n = n - 'A' + 10;
			else if(n >= 'a' && n <= 'f')
				n = n - 'a' + 10;
			else
				break;
			r = n;
			n = s[2];
			if(n >= '0' && n <= '9')
				n = n - '0';
			else if(n >= 'A' && n <= 'F')
				n = n - 'A' + 10;
			else if(n >= 'a' && n <= 'f')
				n = n - 'a' + 10;
			else
				break;
			s += 2;
			c = r*16+n;
		}
		*t++ = c;
	}
	*t = 0;

	/* latin1 heuristic */
	v = malloc(UTFmax*strlen(u) + 1);
	s = u;
	t = v;
	while(*s){
		/* in decoding error, assume latin1 */
		if((n=chartorune(&r, s)) == 1 && r == 0x80)
			r = *s;
		s += n;
		t += runetochar(t, &r);
	}
	*t = 0;

	return v;
}
